#! /usr/bin/python3
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------------------------------------
# Example for basic usage of the hash database
#
# Copyright 2020 Google LLC
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License.  You may obtain a copy of the License at
#     https://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.  See the License for the specific language governing permissions
# and limitations under the License.
#--------------------------------------------------------------------------------------------------

import asyncio
import tkrzw

async def main():
    # Prepares the database.
    dbm = tkrzw.DBM()
    dbm.Open("casket.tkh", True, truncate=True, num_buckets=100)

    # Prepares the asynchronous adapter with 4 worker threads.
    adbm = tkrzw.AsyncDBM(dbm, 4)

    # Executes the Set method asynchronously.
    future = adbm.Set("hello", "world")
    # Does something in the foreground.
    print("Setting a record")
    # Checks the result after awaiting the Set operation.
    # Calling Future#Get doesn't yield the coroutine ownership.
    status = future.Get()
    if status != tkrzw.Status.SUCCESS:
        print("ERROR: " + str(status))

    # Executes the Get method asynchronously.
    future = adbm.GetStr("hello")
    # Does something in the foreground.
    print("Getting a record")
    # Awaits the operation while yielding the coroutine ownership.
    await future
    status, value = future.Get()
    if status == tkrzw.Status.SUCCESS:
        print("VALUE: " + value)

    # Releases the asynchronous adapter.
    adbm.Destruct()

    # Closes the database.
    dbm.Close()

asyncio.run(main())

# END OF FILE



#! /usr/bin/python3
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------------------------------------
# Coroutine tests
#
# Copyright 2020 Google LLC
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License.  You may obtain a copy of the License at
#     https://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.  See the License for the specific language governing permissions
# and limitations under the License.
#--------------------------------------------------------------------------------------------------

import asyncio
import time
import tkrzw

NUM_ITERS = 100
NUM_COROUTINES = 1000
NUM_REPEATS = 100

async def set_sync(dbm, cor_id, iter_count):
  for i in range(0, NUM_REPEATS):
    key = str(cor_id * NUM_COROUTINES * NUM_REPEATS + iter_count * NUM_REPEATS + i)
    dbm.Set(key, key).OrDie()

async def set_async(adbm, cor_id, iter_count):
  futures = []
  for i in range(0, NUM_REPEATS):
    key = str(cor_id * NUM_COROUTINES * NUM_REPEATS + iter_count * NUM_REPEATS + i)
    futures.append(adbm.Set(key, key))
  for future in futures:
    future.Get().OrDie()

async def get_sync(dbm, cor_id, iter_count):
  for i in range(0, NUM_REPEATS):
    key = str(cor_id * NUM_COROUTINES * NUM_REPEATS + iter_count * NUM_REPEATS + i)
    status = tkrzw.Status()
    dbm.Get(key, status)
    status.OrDie()

async def get_async(adbm, cor_id, iter_count):
  futures = []
  for i in range(0, NUM_REPEATS):
    key = str(cor_id * NUM_COROUTINES * NUM_REPEATS + iter_count * NUM_REPEATS + i)
    futures.append(adbm.Get(key))
  for future in futures:
    status, value = future.Get()
    status.OrDie()

async def main():
  dbm = tkrzw.DBM()
  num_buckets = NUM_ITERS * NUM_COROUTINES * NUM_REPEATS / 2
  dbm.Open("casket.tkh", True, concurrent=True, truncate=True,
       num_buckets=num_buckets, file="pos-para")
  adbm = tkrzw.AsyncDBM(dbm, 4)
  def make_set_sync(cor_id, iter_count):
    return set_sync(dbm, cor_id, iter_count)
  def make_set_async(cor_id, iter_count):
    return set_async(adbm, cor_id, iter_count)
  def make_get_sync(cor_id, iter_count):
    return get_sync(dbm, cor_id, iter_count)
  def make_get_async(cor_id, iter_count):
    return get_async(adbm, cor_id, iter_count)
  confs = [
    {"label": "SET SYNC", "op": make_set_sync},
    {"label": "SET ASYNC", "op": make_set_async},
    {"label": "GET SYNC", "op": make_get_sync},
    {"label": "GET ASYNC", "op": make_get_async},
  ]
  for conf in confs:
    start_time = time.time()  
    for iter_count in range(0, NUM_ITERS):
      coroutines = []
      for cor_id in range(0, NUM_COROUTINES):
        coroutines.append(conf["op"](cor_id, iter_count))
      for coroutine in coroutines:
        await coroutine
    end_time = time.time()
    elapsed = end_time - start_time
    print("{:10s}: {:8.0f} QPS".format(
      conf["label"], (NUM_ITERS * NUM_COROUTINES * NUM_REPEATS) / elapsed))
  adbm.Destruct()
  dbm.Close()

asyncio.run(main())

# END OF FILE



#! /usr/bin/python3
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------------------------------------
# Building configurations
#
# Copyright 2020 Google LLC
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License.  You may obtain a copy of the License at
#     https://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.  See the License for the specific language governing permissions
# and limitations under the License.
#--------------------------------------------------------------------------------------------------

import os
import platform
import subprocess

from setuptools import Extension, setup

package_name = 'tkrzw'
package_version = '0.1'
package_description = 'a set of implementations of DBM'
package_author = 'Mikio Hirabayashi'
package_author_email = 'hirarin@gmail.com'
package_url = 'http://dbmx.net/tkrzw/'
module_name = 'tkrzw'

extra_compile_args = ["-std=c++17", "-Wall"]
sources = ['tkrzw.cc']

ld_paths = [
    '/usr/local/lib'
]
os.environ['LD_LIBRARY_PATH'] = os.environ.get('LD_LIBRARY_PATH', '') + ':'.join(ld_paths)


def parse_build_flags(cmd_args):
    result = {}
    output = subprocess.check_output(cmd_args).strip().decode()
    for item in output.split():
        flag = item[0:2]
        result.setdefault(flag, []).append(item[2:])
    return result


# Parse include dirs
flags = parse_build_flags(['tkrzw_build_util', 'config', '-i'])
include_dirs = flags.get('-I', []) or ['/usr/local/include']

# Parse library dirs and libraries
flags = parse_build_flags(['tkrzw_build_util', 'config', '-l'])
library_dirs = flags.get('-L', ['/usr/local/lib'])
libraries = flags.get('-l', [])
if not libraries:
    if platform.system() == "Darwin":
        libraries = ['tkrzw', 'stdc++', 'pthread', 'm', 'c']
    else:
        libraries = ['tkrzw', 'stdc++', 'rt', 'atomic', 'pthread', 'm', 'c']

setup(
    name=package_name,
    version=package_version,
    description=package_description,
    author=package_author,
    author_email=package_author_email,
    url=package_url,
    ext_modules=[
        Extension(
            module_name,
            include_dirs=include_dirs,
            extra_compile_args=extra_compile_args,
            sources=sources,
            library_dirs=library_dirs,
            libraries=libraries
        ),
    ]
)
# END OF FILE



#! /usr/bin/python3
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------------------------------------
# Example for basic usage of the hash database
#
# Copyright 2020 Google LLC
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License.  You may obtain a copy of the License at
#     https://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.  See the License for the specific language governing permissions
# and limitations under the License.
#--------------------------------------------------------------------------------------------------

import tkrzw

# Prepares the database.
dbm = tkrzw.DBM()
dbm.Open("casket.tkh", True, truncate=True, num_buckets=100)

# Sets records.
# If the operation fails, a runtime exception is raised.
# Keys and values are implicitly converted into bytes.
dbm["first"] = "hop"
dbm["second"] = "step"
dbm["third"] = "jump"

# Retrieves record values.
# If the operation fails, a runtime exception is raised.
# Retrieved values are strings if keys are strings.
print(dbm["first"])
print(dbm["second"])
print(dbm["third"])
try:
    print(dbm["fourth"])
except tkrzw.StatusException as e:
    print(repr(e))

# Checks and deletes a record.
if "first" in dbm:
    del dbm["first"]

# Traverses records.
# Retrieved keys and values are always bytes so we decode them.
for key, value in dbm:
    print(key.decode(), value.decode())

# Closes the database.
dbm.Close()

# END OF FILE
